/*
  conffile.cpp
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of wpa_setting.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>

#include "wpa_setting.h"
#include "conffile.h"
#if QWS
#include <qpe/config.h>
#endif
#include <qregexp.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstringlist.h>
#include "sha1pass.h"

ConfFile::ConfFile(QString const &filename)
{
	wlanfilename = filename;
	valid = false;

	scheme = "";
	name = "";
	ssid = "";
	passphrase = "";
	proto_wpa = false;
	scan_ssid = false;

	if (!loadQPEconfig(filename))
		return;

	valid = true;

	if (!loadWPAconfig(scheme))
		return;
}

ConfFile::~ConfFile()
{
}


bool ConfFile::isWlan()
{
	return valid;
}


QString ConfFile::getName()
{
	ASSERT(valid);

	return name;
}

QString ConfFile::getSSID()
{
	ASSERT(valid);
	return ssid;
}

ConfFile::ProtoType ConfFile::getProtocolType()
{
	ASSERT(valid);

#if 0
	if(proto_wpa && proto_wpa2)
		return WPA2_WPA;
#endif
	if(proto_wpa)
		return WPA;
#if 0
	if(proto_wpa2)
		return WPA2;
#endif

	return None_WEP;
}

#if 0
ConfFile::EncType ConfFile::getEncryptType()
{
	ASSERT(valid);
	ASSERT(enc_tkip || enc_ccmp);

	if(enc_tkip)
		return TKIP;
	if(enc_ccmp)
		return AES;
	return AES_TKIP;
}
#endif

QString ConfFile::getProtocolName()
{
	switch(getProtocolType()){
	case None_WEP:
		return QString::fromUtf8("None/WEP");
	case WPA:
#if 0
		return QString::fromUtf8("WPA");
	case WPA2:
		return QString::fromUtf8("WPA2");
	case WPA2_WPA:
#endif
		return QString::fromUtf8("WPA/WPA2");
	default:
		break;
	}
	return QString::null;
}

#if 0
QString ConfFile::getEncryptName()
{
	switch(getEncryptType()){
	case TKIP:
		return QString::fromUtf8("TKIP");
	case AES:
		return QString::fromUtf8("AES");
	case AES_TKIP:
		return QString::fromUtf8("TKIP/AES");
	default:
		break;
	}
	return QString::null;
}
#endif

QString ConfFile::getPassPhrase()
{
	ASSERT(valid);

	return passphrase;
}

bool ConfFile::getStealthSSID()
{
	ASSERT(valid);

	return scan_ssid;
}


bool ConfFile::setSSID(QString const &ssid)
{
	if(!valid)
		return false;

	this->ssid = ssid;
	return true;
}

bool ConfFile::setProtocolType(ProtoType type)
{
	if(!valid)
		return false;

	proto_wpa = false;

	switch(type){
	case WPA:
		proto_wpa = true;
		break;
#if 0
	case WPA2:
		proto_wpa2 = true;
		break;
	case WPA2_WPA:
		proto_wpa = proto_wpa2 = true;
		break;
#endif
	case None_WEP:
	default:
		break;
	}
	return true;
}

#if 0
bool ConfFile::setEncryptType(EncType type)
{
	if(!valid)
		return false;

	enc_tkip = false;
	enc_ccmp = false;

	switch(type){
	case TKIP:
		enc_tkip = true;
		break;
	case AES:
		enc_ccmp = true;
		break;
	case AES_TKIP:
		enc_tkip = enc_ccmp = true;
		break;
	default:
		break;
	}
	return true;
}
#endif

bool ConfFile::setPassPhraseASCII(QString const &pass)
{
	if(!valid)
		return false;

	passphrase = SHA1Pass::getEncodedPass(ssid, pass);
	return true;
}

bool ConfFile::setStealthSSID(bool value)
{
	if(!valid)
		return false;

	scan_ssid = value;
	return true;
}

bool ConfFile::commitChanges()
{
	if(!valid)
		return false;

#if 1
	printf("name:'%s' ssid:'%s' scheme:'%s' pass:'%s' wpa:%d\n",
		   name.ascii(), ssid.ascii(), scheme.ascii(), passphrase.ascii(),
		   proto_wpa);
#endif

	saveWLANconfig();
	saveQPEconfig();
	saveWPAconfig();

	return true;
}

QString ConfFile::convHexToBin(QString const &hex)
{
	QString result;
	QStringList list = QStringList::split(QString(":"), hex);
	unsigned int i;

	for(i = 0; i < list.count(); i++){
		result.append((char)strtoul(list[i].ascii(), NULL, 16));
	}

	return result;
}

QString ConfFile::convBinToHex(QString const &bin)
{
	return bin;
}

QString ConfFile::escapeAndQuote(QString const &str)
{
	int i;
	QString result("");

	for( i = 0; i < (int)str.length(); i++){
		switch(str[i]){
		case '"':
		case '$':
			result.append('\\');
			// FALL THROUGH
		default:
			result.append(str[i]);
			break;
		}
	}
	return "\""+result+"\"";
}

bool ConfFile::loadQPEconfig(QString const &file)
{
	Config conf(file, Config::File);

	if(!conf.isValid())
		return false;

	conf.setGroup(QString("Info"));
	if(conf.readEntry("Type") != "wlan")
		return false;

	name = conf.readEntry("Name");
	scheme = conf.readEntry("Scheme");

	conf.setGroup(QString("Properties"));

	ssid = conf.readEntry("essid");

	proto_wpa = false;

	return true;
}

bool ConfFile::loadWPAconfig(QString const &scheme)
{
	bool del = false;
	QFile wpa_conf(QString(WPA_CONF_PATH "%1.conf").arg(scheme));

	if (!wpa_conf.exists())
		return false;

	wpa_conf.open(IO_ReadOnly);

	proto_wpa = true;

	while (!wpa_conf.atEnd()) {
		QString line;
		wpa_conf.readLine(line, 256);

		// strip \r\n
		line.stripWhiteSpace();
		while(line.right(1).contains(QRegExp("[ \r\n]"))){
			line = line.mid(0, line.length() - 1);
		}

		int comment = line.find('#');
		if (comment > 0) {
			line = line.left(comment);
		}

		// get ssid
		if (line.contains(QRegExp("ssid=\"")) > 0) {
			QString wpa_ssid = line.mid(line.find('=') + 1);
			wpa_ssid.stripWhiteSpace();
			if (wpa_ssid.left(1) == "\"" && wpa_ssid.right(1) == "\"")
				wpa_ssid = wpa_ssid.mid(1, wpa_ssid.length() - 2);

			if (wpa_ssid != ssid)
				del = true;

			ssid = wpa_ssid;
		}

		// get encoded passphrase
		if (line.contains(QRegExp("psk=")) > 0) {
			passphrase = line.mid(line.find('=') + 1);
			passphrase.stripWhiteSpace();
		}

		// get encoded passphrase
		if (line.contains(QRegExp("scan_ssid\\s*=\\s*1$")) > 0) {
			printf("scan\n");
			scan_ssid = true;
		}
	}

	if (del) {
		wpa_conf.remove();
		proto_wpa = false;
	}

#if 1
	printf("name:'%s' ssid:'%s' scheme:'%s' pass:'%s' wpa:%d\n",
		   name.ascii(), ssid.ascii(), scheme.ascii(), passphrase.ascii(),
		   proto_wpa);
#endif

	return true;
}

void ConfFile::saveWLANconfig()
{
	QString line;
	QString scm;

	QFile org("/etc/pcmcia/wlan-ng.opts");
	QFile mod("/tmp/wlan-ng.opts");

	if(!valid)
		return;

	if (!org.exists()
		|| !org.open(IO_ReadOnly)
		|| !mod.open(IO_WriteOnly)) {
		org.close();
		mod.close();
		return;
	}

	QTextStream in(&org);
	QTextStream out(&mod);

	while (!in.eof()) {
		line = in.readLine();
	
		if(line.contains(QRegExp("^.+,\\*,\\*,\\*)"))){
			QStringList array = QStringList::split(',', line);
			scm = array[0];
		}
		else if(line.contains(QRegExp("DesiredSSID=\".*\""))){
			if(scm == scheme){
				line = "    DesiredSSID=\"" + ssid + "\"";
			}
		}
		out << line + "\n";
	}

	org.close();
	mod.close();

	system(QString("su -c 'mv %1 %2; chown root.qpe %3'").
		   arg(mod.name()).arg(org.name()).arg(org.name()).ascii());
}

void ConfFile::saveQPEconfig()
{
	Config conf(wlanfilename, Config::File);

	if(!conf.isValid())
		return;

	conf.setGroup(QString("Info"));
	if(conf.readEntry("Type") != "wlan")
		return;

	conf.setGroup(QString("Properties"));

	conf.writeEntry("essid", ssid);
}

void ConfFile::saveWPAconfig()
{
	/* Create wpa config file */
	QFile wpa_conf(QString(WPA_CONF_PATH "%1.conf").arg(scheme));

	printf("file:%s\n", wpa_conf.name().ascii());

	if (!proto_wpa) {
		/* if dont need wpa_supplicant, remove conf file */
		wpa_conf.remove();
		return;
	}

	if (!wpa_conf.open(IO_WriteOnly)) {
		return;
	}

	QTextStream out(&wpa_conf);

	out << "network={\n";

	if(scan_ssid)
		out << "\tscan_ssid=1\n";

	out << QString("\tssid=\"%1\"\n").arg(ssid)
		<< QString("\tpsk=%1\n").arg(passphrase)
		<< "}\n";

	wpa_conf.close();
}
